@@ -139,6 +139,7 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_IPFW_NOSUPPORT);
         CASE_CODE (SC_ERR_IPFW_BIND);
         CASE_CODE (SC_ERR_IPFW_SOCK);
+        CASE_CODE (SC_ERR_IPFW_SETSOCKOPT);
         CASE_CODE (SC_ERR_IPFW_NOPORT);
         CASE_CODE (SC_WARN_IPFW_RECV);
         CASE_CODE (SC_WARN_IPFW_XMIT);
@@ -155,6 +156,7 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_NFQ_NOSUPPORT);
         CASE_CODE (SC_ERR_NFQ_UNBIND);
         CASE_CODE (SC_ERR_NFQ_BIND);
+        CASE_CODE (SC_ERR_NFQ_HANDLE_PKT);
         CASE_CODE (SC_ERR_CUDA_ERROR);
         CASE_CODE (SC_ERR_CUDA_HANDLER_ERROR);
         CASE_CODE (SC_ERR_TM_THREADS_ERROR);
@@ -238,6 +240,8 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_EVENT_ENGINE);
         CASE_CODE (SC_ERR_NO_LUAJIT_SUPPORT);
         CASE_CODE (SC_ERR_LUAJIT_ERROR);
+        CASE_CODE (SC_ERR_NO_GEOIP_SUPPORT);
+        CASE_CODE (SC_ERR_GEOIP_ERROR);
         CASE_CODE (SC_ERR_DEFRAG_INIT);
         CASE_CODE (SC_ERR_NO_REPUTATION);
         CASE_CODE (SC_ERR_NOT_SUPPORTED);
