@@ -255,6 +255,9 @@ typedef enum {
     SC_ERR_NAPATECH_PARSE_CONFIG,
     SC_ERR_NO_REPUTATION,
     SC_ERR_NOT_SUPPORTED,
+    SC_ERR_IPFW_SETSOCKOPT,
+    SC_ERR_NO_GEOIP_SUPPORT,
+    SC_ERR_GEOIP_ERROR,
 } SCError;
 
 const char *SCErrorToString(SCError);
